<?php
/**
 * The plugin creation file.
 *
 * @package Uli_Reading_Time_WP
 *
 * Plugin Name: Uli Reading Time WP
 * Plugin URI: https://ullisesfreitas.com/plugins/uli-reading-time
 * Description: Add an estimated reading time to your posts.
 * Version: 1.0.0
 * Author: Ulises Freitas
 * Author URI: https://ulisesfreitas.com
 * License: GPL2
 * Text Domain: ui-reading-time-wp
 * Domain Path: /languages
 *
 */

// If this file is called directly, abort.
if ( ! defined( 'WPINC' ) ) {
	die;
}

/**
 * Class for calculating reading time.
 *
 * The class that contains all functions for calculating reading time.
 *
 * @since 1.0.0
 */
class Uli_Reading_Time_WP {

	/**
	 * Add label option using add_option if it does not already exist.
	 *
	 * @var string
	 */
	public $reading_time;

	/**
	 * Allowed HTML tags for setting fields.
	 *
	 * @var array
	 */
	public $rtwp_kses = array(
		'br'     => array(),
		'em'     => array(),
		'b'      => array(),
		'strong' => array(),
	);

	/**
	 * Construct function for Uli Reading Time WP.
	 *
	 * Create default settings on plugin activation. Add shortcode. Add options.
	 * Add menu page.
	 *
	 * @since 1.0.0
	 */
	public function __construct() {
		load_plugin_textdomain( 'uli-reading-time-wp', false, basename( dirname( __FILE__ ) ) . '/languages/' );

		$default_settings = array(
			'label'              => __( 'Reading Time: ', 'uli-reading-time-wp' ),
			'postfix'            => __( 'minutes', 'uli-reading-time-wp' ),
			'postfix_singular'   => __( 'minute', 'uli-reading-time-wp' ),
			'wpm'                => 300,
			'before_content'     => true,
			'before_excerpt'     => true,
			'exclude_images'     => false,
			'include_shortcodes' => false,
		);

		$ulirt_post_type_args = array(
			'public' => true,
		);

		$ulirt_post_type_args = apply_filters( 'ulirt_post_type_args', $ulirt_post_type_args );

		$ulirt_post_types = get_post_types( $ulirt_post_type_args );

		foreach ( $ulirt_post_types as $rtwp_post_type ) {
			if ( 'attachment' === $rtwp_post_type ) {
				continue;
			}
			$default_settings['post_types'][ $rtwp_post_type ] = true;
		}

		$uli_rt_reading_time_options = get_option( 'uli_rt_reading_time_options' );

		add_shortcode( 'ulirt_reading_time', array( $this, 'ulirt_reading_time' ) );
		add_option( 'uli_rt_reading_time_options', $default_settings );
		add_action( 'admin_menu', array( $this, 'uli_rt_reading_time_admin_actions' ) );

		add_action('admin_head', array( $this, 'ulirt_load_admin_jscss' ));

		$remove_from_yoast = isset($uli_rt_reading_time_options['remove_from_yoast'] ) ? $this->ulirt_convert_boolean( $uli_rt_reading_time_options['remove_from_yoast'] ) : false;
		if(isset($remove_from_yoast) && true === $remove_from_yoast)
		{
			add_filter( 'wpseo_opengraph_desc', array($this, 'remove_reading_time_from_yoast') );
		}
		
		$rt_before_content = isset($uli_rt_reading_time_options['before_content'] ) ? $this->ulirt_convert_boolean( $uli_rt_reading_time_options['before_content'] ) : false;

		if ( isset( $rt_before_content ) && true === $rt_before_content ) {
			add_filter( 'the_content', array( $this, 'ulirt_add_reading_time_before_content' ) );
		}

		$rt_before_excerpt = isset( $uli_rt_reading_time_options['before_excerpt'] ) ? $this->ulirt_convert_boolean( $uli_rt_reading_time_options['before_excerpt'] ) : false;

		if ( isset( $rt_before_excerpt ) && true === $rt_before_excerpt ) {
			add_filter( 'get_the_excerpt', array( $this, 'ulirt_add_reading_time_before_excerpt' ), 1000 );
		}

		

	}

	/**
	 * Calculate the reading time of a post.
	 *
	 * Gets the post content, counts the images, strips shortcodes, and strips tags.
	 * Then counts the words. Converts images into a word count. And outputs the
	 * total reading time.
	 *
	 * @since 1.0.0
	 *
	 * @param int   $rt_post_id The Post ID.
	 * @param array $rt_options The options selected for the plugin.
	 * @return string|int The total reading time for the article or string if it's 0.
	 */
	public function ulirt_calculate_reading_time( $rt_post_id, $rt_options ) {

		$rt_content       = get_post_field( 'post_content', $rt_post_id );
		$number_of_images = substr_count( strtolower( $rt_content ), '<img ' );

		if ( ! isset( $rt_options['include_shortcodes'] ) ) {
			$rt_content = strip_shortcodes( $rt_content );
		}

		$rt_content = wp_strip_all_tags( $rt_content );
		$word_count = count( preg_split( '/\s+/', $rt_content ) );

		if ( isset( $rt_options['exclude_images'] ) && ! $rt_options['exclude_images'] ) {
			// Calculate additional time added to post by images.
			$additional_words_for_images = $this->ulirt_calculate_images( $number_of_images, $rt_options['wpm'] );
			$word_count                 += $additional_words_for_images;
		}

		$word_count = apply_filters( 'rtwp_filter_wordcount', $word_count );

		$this->reading_time = $word_count / $rt_options['wpm'];

		// If the reading time is 0 then return it as < 1 instead of 0.
		if ( 1 > $this->reading_time ) {
			$this->reading_time = __( '< 1', 'uli-reading-time-wp' );
		} else {
			$this->reading_time = ceil( $this->reading_time );
		}

		return $this->reading_time;

	}

	/**
	 * Adds additional reading time for images
	 *
	 * Calculate additional reading time added by images in posts. Based on calculations by Medium. https://blog.medium.com/read-time-and-you-bc2048ab620c
	 *
	 * @since 1.1.0
	 *
	 * @param int   $total_images number of images in post.
	 * @param array $wpm words per minute.
	 * @return int  Additional time added to the reading time by images.
	 */
	public function ulirt_calculate_images( $total_images, $wpm ) {
		$additional_time = 0;
		// For the first image add 12 seconds, second image add 11, ..., for image 10+ add 3 seconds.
		for ( $i = 1; $i <= $total_images; $i++ ) {
			if ( $i >= 10 ) {
				$additional_time += 3 * (int) $wpm / 60;
			} else {
				$additional_time += ( 12 - ( $i - 1 ) ) * (int) $wpm / 60;
			}
		}

		return $additional_time;
	}

	/**
	 * Output the proper postfix for the reading time.
	 *
	 * @since 2.0.5
	 *
	 * @param string|int $time The total reading time for the article or string if it's 0.
	 * @param string     $singular The postfix singular label.
	 * @param string     $multiple The postfix label.
	 *
	 * @return string $postfix The calculated postfix.
	 */
	public function uli_rt_add_postfix( $time, $singular, $multiple ) {
		if ( (int) $time > 1 ) {
			$postfix = $multiple;
		} else {
			$postfix = $singular;
		}

		$postfix = apply_filters( 'rt_edit_postfix', $postfix, $time, $singular, $multiple );

		return $postfix;
	}

	/**
	 * Creates the [ulirt_reading_time] shortcode.
	 *
	 * @since 1.0.0
	 *
	 * @param array  $atts The attributes of the shortcode.
	 * @param string $content The content of the shortcode.
	 * @return string.
	 */
	public function ulirt_reading_time( $atts, $content = null ) {

		$atts = shortcode_atts(
			array(
				'label'            => '',
				'postfix'          => '',
				'postfix_singular' => '',
				'post_id'          => '',
			),
			$atts,
			'ulirt_reading_time'
		);

		$uli_rt_reading_time_options = get_option( 'uli_rt_reading_time_options' );

		// If post_id attribute was specified that exists, then use that to calculate read time, else use the current post ID.
		$rt_post = $atts['post_id'] && ( get_post_status( $atts['post_id'] ) ) ? $atts['post_id'] : get_the_ID();

		$this->ulirt_calculate_reading_time( $rt_post, $uli_rt_reading_time_options );

		$calculated_postfix = $this->uli_rt_add_postfix( $this->reading_time, $atts['postfix_singular'], $atts['postfix'] );

		return '<span class="span-reading-time ulirt-reading-time"><span class="rt-label rt-prefix">' . wp_kses( $atts['label'], $this->rtwp_kses ) . '</span> <span class="rt-time"> ' . esc_html( $this->reading_time ) . '</span> <span class="rt-label rt-postfix">' . wp_kses( $calculated_postfix, $this->rtwp_kses ) . '</span></span>';
	}

	/**
	 * Include the Reading Time Admin page.
	 *
	 * The reading-time-admin.php contains everything needed to handle
	 * the options in the admin screen.
	 *
	 * @since 1.0.0
	 */
	public function uli_rt_reading_time_admin() {
		include 'uli-rt-reading-time-admin.php';
	}

	/**
	 * Create the options page for the admin screen.
	 *
	 * @since 1.0.0
	 */
	public function uli_rt_reading_time_admin_actions() {
		add_options_page(
			__( 'Uli Reading Time WP Settings', 'uli-reading-time-wp' ),
			__( 'Uli Reading Time WP', 'uli-reading-time-wp' ),
			'manage_options',
			'uli-rt-reading-time-settings',
			array( $this, 'uli_rt_reading_time_admin' )
		);
	}

	/**
	 * Adds the reading time before the_content.
	 *
	 * If the option is selected to automatically add the reading time before
	 * the_content, the reading time is calculated and added to the beginning of the_content.
	 *
	 * @since 1.0.0
	 *
	 * @param string $content The original post content.
	 * @return string The post content with reading time prepended.
	 */
	public function ulirt_add_reading_time_before_content( $content ) {
		$uli_rt_reading_time_options = get_option( 'uli_rt_reading_time_options' );

		// Get the post type of the current post.
		$rtwp_current_post_type = get_post_type();

		if ( ! isset( $uli_rt_reading_time_options['post_types'] ) ) {
			$uli_rt_reading_time_options['post_types'] = array();
		}

		// If the current post type isn't included in the array of post types or it is and set to false, don't display it.
		if ( ! isset( $uli_rt_reading_time_options['post_types'][ $rtwp_current_post_type ] ) || ! $uli_rt_reading_time_options['post_types'][ $rtwp_current_post_type ] ) {
			return $content;
		}

		$original_content = $content;
		$rt_post          = get_the_ID();

		$this->ulirt_calculate_reading_time( $rt_post, $uli_rt_reading_time_options );

		$label            = $uli_rt_reading_time_options['label'];
		$postfix          = $uli_rt_reading_time_options['postfix'];
		$postfix_singular = $uli_rt_reading_time_options['postfix_singular'];

		if ( in_array( 'get_the_excerpt', $GLOBALS['wp_current_filter'], true ) ) {
			return $content;
		}

		$calculated_postfix = $this->uli_rt_add_postfix( $this->reading_time, $postfix_singular, $postfix );

		$content  = '<span class="rt-reading-time" style="display: block;"><span class="rt-label rt-prefix">' . wp_kses( $label, $this->rtwp_kses ) . '</span> <span class="rt-time">' . esc_html( $this->reading_time ) . '</span> <span class="rt-label rt-postfix">' . wp_kses( $calculated_postfix, $this->rtwp_kses ) . '</span></span>';
		$content .= $original_content;
		return $content;
	}

	/**
	 * Adds the reading time before the_excerpt.
	 *
	 * If the options is selected to automatically add the reading time before
	 * the_excerpt, the reading time is calculated and added to the beginning of the_excerpt.
	 *
	 * @since 1.0.0
	 *
	 * @param string $content The original content of the_excerpt.
	 * @return string The excerpt content with reading time prepended.
	 */
	public function ulirt_add_reading_time_before_excerpt( $content ) {
		$uli_rt_reading_time_options = get_option( 'uli_rt_reading_time_options' );

		// Get the post type of the current post.
		$rtwp_current_post_type = get_post_type();

		if ( ! isset( $uli_rt_reading_time_options['post_types'] ) ) {
			$uli_rt_reading_time_options['post_types'] = array();
		}

		// If the current post type isn't included in the array of post types or it is and set to false, don't display it.
		if ( ! isset( $uli_rt_reading_time_options['post_types'][ $rtwp_current_post_type ] ) || ! $uli_rt_reading_time_options['post_types'][ $rtwp_current_post_type ] ) {
			return $content;
		}

		$original_content = $content;
		$rt_post          = get_the_ID();

		$this->ulirt_calculate_reading_time( $rt_post, $uli_rt_reading_time_options );

		$label            = $uli_rt_reading_time_options['label'];
		$postfix          = $uli_rt_reading_time_options['postfix'];
		$postfix_singular = $uli_rt_reading_time_options['postfix_singular'];

		$calculated_postfix = $this->uli_rt_add_postfix( $this->reading_time, $postfix_singular, $postfix );

		$content  = '<span class="rt-reading-time" style="display: block;"><span class="rt-label rt-prefix">' . wp_kses( $label, $this->rtwp_kses ) . '</span> <span class="rt-time">' . esc_html( $this->reading_time ) . '</span> <span class="rt-label rt-postfix">' . wp_kses( $calculated_postfix, $this->rtwp_kses ) . '</span></span> ';
		$content .= $original_content;
		return $content;
	}

	/**
	 * A function to fix some bad legacy code using a string for true and false.
	 *
	 * @param string $value A string set to either 'true' or 'false'.
	 */
	public function ulirt_convert_boolean( $value ) {
		if ( 'true' === $value || true === $value ) {
			return true;
		} else {
			return false;
		}
	}

	# load wp-admin css and js files
	public function ulirt_load_admin_jscss() {
		wp_enqueue_script('postbox');
		wp_enqueue_style('ulirt-styles', plugins_url('assets/css/styles.css', __FILE__), array(), filemtime(plugin_dir_path( __FILE__).'assets/css/styles.css'));
		//wp_enqueue_script('uli_minify-min', plugins_url('uli_minify.js', __FILE__), array('jquery'), filemtime(plugin_dir_path( __FILE__).'uli_minify.js'), true);
	}


	/**
	 * A Function to remove the auto generated Reading Time from Yoast og:desc
	 *
	 * This hooks into Yoast's `wpseo_opengraph_desc` filter and uses a regular
	 * expression to remove the reading time for the post
	 * 
	 * @see Reading Time WP Plugin
	 * @link https://wordpress.org/plugins/reading-time-wp/
	 *
	 * @param string $ogdesc The excerpt grabbed by Yoast for output.
	 * @return string The $ogdesc with the Reading Time string removed.
	 */
	public function remove_reading_time_from_yoast( $ogdesc ) {
		
		//First we get the options set on this version of Reading Time WP
		$rtwp_options = get_option('uli_rt_reading_time_options');
		$rtwp_label = $rtwp_options['label'];
		$rtwp_postfix = $rtwp_options['postfix'];
		$rtwp_singular = $rtwp_options['postfix_singular'];

		// Next we build a regular expression using those options
		$rtwp_regex = $rtwp_label . '(.*?)(' . $rtwp_postfix . '|' . $rtwp_singular . ')';
		
		// Finally we use `preg_replace` to replace the reading time with an empty string
		$ogdesc = preg_replace('/' . $rtwp_regex . '/', '', $ogdesc);

		return $ogdesc;	
	}

}

function ulirt_init() {
	global $reading_time_wp;
	$reading_time_wp = new Uli_Reading_Time_WP();
}
add_action( 'init', 'ulirt_init' );
